"""
Field
"""

from django.conf import settings
from django.contrib.admin import widgets as admin_widgets
from django.db.models import TextField

from .widgets import ProseMirrorWidget


class ProseMirrorField(TextField):
    """
    ProseMirror field
    """
    def __init__(self, *args, **kwargs):
        """
        Constructor
        """
        options = kwargs.pop("prosemirror_profile", "default")
        self.widget = ProseMirrorWidget(prosemirror_profile=options)
        super(ProseMirrorField, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        """
        Construct form field
        """
        defaults = {"widget": self.widget}
        defaults.update(kwargs)

        if defaults["widget"] == admin_widgets.AdminTextareaWidget:
            defaults["widget"] = self.widget
        return super(ProseMirrorField, self).formfield(**defaults)



"""
Fields
"""

import json

try:
    from django.urls import reverse_lazy
except ImportError:
    # Django <1.10 compliance
    from django.core.urlresolvers import reverse_lazy

from django.conf import settings
from django.contrib.staticfiles.storage import staticfiles_storage
from django import forms
from django.forms import widgets

MAXI = {
    "type": "bar",  # can be bar or tooltip
    "schema": "maxi",  # can be maxi or micro
}

MICRO = {
    "type": "tooltip",  # can be bar or tooltip
    "schema": "micro",  # can be maxi or micro
}

DEFAULTS = {
    "GLOBAL": {
        "inline_code": False,  # allows inline code
        "rule": False,  # Allows rule
    },
    "PROFILES": {
        "default": MAXI,
        "maxi": MAXI,
        "micro": MICRO,
    },
}
CONFIG = getattr(settings, "PROSEMIRROR", {})


class ProseMirrorWidget(widgets.Textarea):
    """
    ProseMirror widget
    """
    def __init__(self, *args, **kwargs):
        """
        Constructor
        """
        self.profile = kwargs.pop("prosemirror_profile", "default")
        attrs = kwargs.get("attrs", {})
        attrs["data-prosemirror-options"] = json.dumps(self.options)
        attrs["class"] = (
            "%s prosemirror-box" % attrs.get("class", "")).strip()
        kwargs["attrs"] = attrs
        super(ProseMirrorWidget, self).__init__(*args, **kwargs)

    @property
    def options(self):
        """
        Get options
        """
        config = DEFAULTS.copy()
        config.update(CONFIG)
        options = config["GLOBAL"].copy()
        options.update(config["PROFILES"][self.profile])
        return options

    def _media(self):
        """
        Include media
        """
        js = (
            staticfiles_storage.url("prosemirror/widget.min.js"),
        )

        css = {
            "all": (
                staticfiles_storage.url(
                    "prosemirror/widget.min.css"),
            )
        }
        return forms.Media(css=css, js=js)
    media = property(_media)



"""
Install script
"""

import codecs
import os
import re

from setuptools import setup, find_packages

import versioneer


###############################################################################

NAME = "django-prosemirror"
AUTHOR = "Tom Van Damme"
EMAIL = "t_o_mvandamme@hotmail.com"
URL = "https://github.com/zakdoek/django-prosemirror/"
LICENSE = "MIT"
DESCRIPTION = "Prosemirror field for django"
PACKAGES = find_packages()
CLASSIFIERS = [
    "Framework :: Django",
]
INSTALL_REQUIRES = [
    "commonmark>=0.7.0",
]

###############################################################################

README = open(os.path.join(os.path.dirname(__file__), "README.md")).read()


if __name__ == "__main__":
    print(PACKAGES)
    setup(
        name=NAME,
        author=AUTHOR,
        author_email=EMAIL,
        url=URL,
        license=LICENSE,
        description=DESCRIPTION,
        long_description=README,
        version=versioneer.get_version(),
        cmdclass=versioneer.get_cmdclass(),
        packages=PACKAGES,
        include_package_data=True,
        classifiers=CLASSIFIERS,
        install_requires=INSTALL_REQUIRES,
    )



"""
Commonmark with header level extension
"""

from CommonMark.blocks import Parser
from CommonMark.render.html import HtmlRenderer


class SteakRenderer(HtmlRenderer):
    """
    Custom renderer
    """
    def __init__(self, *args, **kwargs):
        """
        Set options
        """
        self._headerleveloffset = kwargs.pop("headerleveloffset", 0)
        super(SteakRenderer, self).__init__(*args, **kwargs)

    def heading(self, node, entering):
        """
        Override header level
        """
        node.level = min(node.level + self._headerleveloffset, 6)
        super(SteakRenderer, self).heading(node, entering)


def render(source, headerleveloffset=0):
    """
    Render markdown with Steak Renderer
    """
    parser = Parser()
    ast = parser.parse(source)
    renderer = SteakRenderer(headerleveloffset=headerleveloffset)
    return renderer.render(ast)



"""
General template library
"""

from django import template
from django.utils.safestring import mark_safe

from ..steakmark import render

register = template.Library()  # pylint: disable=invalid-name


@register.simple_tag
def prosemirror(content, headingoffset=0):
    """
    Render prosemirror content as html with a specified header level.

    1 is h1, 2 is h2, ...
    """
    return mark_safe(render(content, headingoffset))
